<?php

use LearnDash\Achievements\Achievement;

class Immerseus_Calendar_Achievements_Connector {
	const COURSE_PLANNER_META = 'imm_calendar_event_course_planner';

	/**
	 * Process achievement when user completes a lesson
	 *
	 * @param array $data Lesson data with array keys:
	 *                    'user' (WP_User object),
	 *                    'course' (post object),
	 *                    'lesson' (post object),
	 *                    'progress' (array).
	 */
	public function add_achievement( array $data ) {
		if ( ! $this->learndash_achievements_is_active() or ! class_exists( Achievement::class ) ) {
			return;
		}

		$start_date = strtotime( 'today' );
		$end_of_day = strtotime( 'tomorrow', $start_date ) - 1;

		$course_id = $data['course']->ID;
		$user_id   = get_current_user_id();

		// Check if the lesson completed date matches the event date
		$post_ids_created_by_student = get_posts( [
			'post_type'      => 'imm_calendar_event',
			'posts_per_page' => -1,
			'fields'         => 'ids',
			'post_status'    => 'private',
			'author'         => $user_id,
			'meta_query'     => [
				'relation' => 'AND',
				[
					'key'     => self::COURSE_PLANNER_META,
					'compare' => '=',
					'value'   => $course_id,
				],
				[
					'key'     => 'imm_calendar_event_start_date',
					'compare' => 'BETWEEN',
					'value'   => [
						date( 'Y-m-d H:i:s', $start_date ),
						date( 'Y-m-d H:i:s', $end_of_day)
					],
					'type'    => 'DATETIME',
				],
			],
		] );

		if ( empty( $post_ids_created_by_student ) ) {
			return;
		}

		foreach ( $post_ids_created_by_student as $post_id ) {
			Achievement::new(
				'complete_planned_event',
				$user_id,
				null,
				null,
				$course_id,
				null,
				null,
				null
			);
		}
	}

	public function add_trigger( array $triggers ): array {
		$triggers['LearnDash Calendar'] = [
			'complete_planned_event' => 'User completes planned calendar event',
		];

		return $triggers;
	}

	public function add_field() {
		if ( ! $this->learndash_achievements_is_active() ) {
			return;
		}

		$course_list = wp_list_pluck(
			get_posts( [
				'post_type' => 'sfwd-courses',
				'post__in'  => learndash_get_user_course_access_list( get_current_user_id() ),
			] ),
			'post_title',
			'ID'
		);

		$input = '<select name="course_planner" id="imm_event_form_course_planner_input" style="width: 100%%;" />';
		$input .= sprintf( '<option value="">%s</option>', __( 'Select', 'immerseus-calendar' ) );
		foreach ( $course_list as $option_value => $option_label ) {
			$input .= sprintf(
				'<option value="%s">%s</option>',
				esc_attr( $option_value ),
				esc_html( $option_label )
			);
		}
		$input .= '</select>';

		echo sprintf(
			'
	            <div class="imm-event-form-field">
	                <label class="imm-event-form-field-label" for="imm_event_form_course_planner_input">
						%s
	                </label>
	                %s
	            </div>
	        ',
			__( 'Course Planner', 'immerseus-calendar' ),
			$input
		);
	}

	public function save_field( int $post_id, array $form_data ) {
		if ( ! $this->learndash_achievements_is_active() ) {
			return;
		}

		if ( ! empty( $form_data['course_planner'] ) ) {
			update_post_meta(
				$post_id,
				self::COURSE_PLANNER_META,
				intval( $form_data['course_planner'] )
			);
		} else {
			delete_post_meta( $post_id, self::COURSE_PLANNER_META );
		}
	}

	public function add_prop_to_calendar_object( int $post_id, Immerseus_Calendar_Event_DTO $event_DTO ) {
		if ( ! $this->learndash_achievements_is_active() ) {
			return;
		}

		$course_planner_id = (string) get_post_meta( $post_id, self::COURSE_PLANNER_META, true );

		$event_DTO->set_extended_props( $course_planner_id, 'coursePlanner', true );

		$event_DTO->set_extended_props( get_the_title( $course_planner_id ), 'coursePlannerTitle', true );
	}

	public function enqueue_scripts() {
		if ( ! $this->learndash_achievements_is_active() ) {
			return;
		}

		wp_add_inline_script(
			'immerseus-calendar-shortcode',
			'
				jQuery(document).on("on_edit_learndash_calendar_event", function() {
		            jQuery("#imm_event_form_course_planner_input").val(window.current_event.extendedProps.coursePlanner);
				});
				
				jQuery(document).on("on_show_learndash_calendar_event", function() {
					if (window.current_event.extendedProps.coursePlanner) {
			            jQuery("#event-view-modal .modal__content").append(
			                "<p style=\"margin-top: 10px;\">" + 
			                	"' . esc_html( __( 'Course Planner', 'immerseus-calendar' ) ) . ': " + 
			                	window.current_event.extendedProps.coursePlannerTitle + 
		                    "</p>"
			            );
		            }
				});
			'
		);
	}

	private function learndash_achievements_is_active(): bool {
		return class_exists( 'LearnDash_Achievements' );
	}
}
