<?php

class Immerseus_Calendar_Events_Mapper {
	public function map( int $user_id ): array {
		if ( $user_id === 0 ) {
			return $this->map_calendar_events( $user_id );
		}

		return array_merge(
			$this->map_lessons_for_students( $user_id ),
			$this->map_calendar_events( $user_id ),
			$this->map_courses_and_quizzes( $user_id )
		);
	}

	/**
	 * Maps lessons with "Lesson Release Schedule" option enabled.
	 *
	 * @return array
	 */
	protected function map_lessons_for_students( $user_id ): array {
		$lessons = $this->get_lessons_by_user( $user_id );

		if ( empty( $lessons ) ) {
			return [];
		}

		$events = [];

		foreach ( $lessons as $lesson_id => $lesson_title ) {
			$lesson_access_from_timestamp = ld_lesson_access_from( $lesson_id, $user_id );

			if ( empty( $lesson_access_from_timestamp ) ) {
				continue;
			}

			$event = new Immerseus_Calendar_Event_DTO(
				$lesson_id,
				$lesson_title . ' ' . __( 'becomes available', 'immerseus-calendar' ),
				get_date_from_gmt(
					date( 'Y-m-d H:i:s', $lesson_access_from_timestamp ),
					'Y-m-d\TH:i:s'
				)
			);

			$events[] = $event->to_array();
		}

		return $events;
	}

	protected function map_calendar_events( int $user_id ): array {
		$query = $this->map_calendar_events_query( $user_id );

		if ( is_null( $query ) ) {
			return [];
		}

		$events = [];

		while ( $query->have_posts() ) {
			$query->the_post();

			$post_id = get_the_ID();
			$start   = get_date_from_gmt(
				get_post_meta( $post_id, 'imm_calendar_event_start_date', true ),
				'Y-m-d\TH:i:s'
			);
			$all_day = (bool) get_post_meta( $post_id, 'imm_calendar_event_all_day', true );

			// Create event

			$event = new Immerseus_Calendar_Event_DTO( $post_id, get_the_title(), $start );

			if ( ! $all_day ) {
				$end = get_date_from_gmt(
					get_post_meta( $post_id, 'imm_calendar_event_end_date', true ),
					'Y-m-d\TH:i:s'
				);

				$event->set_end( $end );
			}

			$event->set_all_day( $all_day );

			$event->set_background_color(
				get_post_meta( $post_id, 'imm_calendar_event_background_color', true )
			);

			$event->set_text_color(
				get_post_meta( $post_id, 'imm_calendar_event_text_color', true )
			);

			$event->set_extended_props(
				(string) get_post_meta( $post_id, 'imm_calendar_event_link', true ),
				'url'
			);

			$event->set_extended_props(
				get_the_author_meta( 'ID' ) == get_current_user_id(),
				'isAuthor'
			);

			$events[] = $event->to_array();
		}

		wp_reset_postdata();

		return $events;
	}

	protected function map_courses_and_quizzes( int $user_id ): array {
		$query = $this->map_courses_and_quizzes_query( $user_id );

		if ( is_null( $query ) ) {
			return [];
		}

		$events = [];

		while ( $query->have_posts() ) {
			$query->the_post();

			$post_id = get_the_ID();

			$event = new Immerseus_Calendar_Event_DTO(
				$post_id,
				get_the_title() . ' ' . __( 'becomes unavailable', 'immerseus-calendar' ),
				get_date_from_gmt(
					get_post_meta( $post_id, 'imm_calendar_due_date', true ),
					'Y-m-d\TH:i:s'
				)
			);

			$events[] = $event->to_array();
		}

		wp_reset_postdata();

		return $events;
	}

	/**
	 * @param int $user_id
	 *
	 * @return WP_Query|null
	 */
	protected function map_calendar_events_query( int $user_id ) {
		$query_args = [
			'post_type'      => 'imm_calendar_event',
			'posts_per_page' => -1,
			'meta_query'     => [
				[
					'key'     => 'imm_calendar_event_start_date',
					'compare' => 'EXISTS',
				],
			],
		];

		if ( learndash_is_admin_user( $user_id ) ) {
			// Admins see all events except students' private events
			$query_args['post_status'] = 'publish';
			$query_args['author__in']  = get_users( [
				'fields'   => 'ID',
				'role__in' => [ 'administrator', 'group_leader' ],
			] );
		} elseif ( learndash_is_group_leader_user( $user_id ) ) {
			// And events assigned to them via their group

			$post_ids_with_group_leader_groups = [];

			foreach ( learndash_get_users_group_ids( $user_id ) as $group_id ) {
				$post_ids_with_group_leader_groups = array_merge(
					$post_ids_with_group_leader_groups,
					get_posts( array_merge( $query_args, [
						'fields'      => 'ids',
						'post_status' => 'publish',
						'meta_query'  => [
							'relation' => 'AND',
							[
								'key'   => 'imm_calendar_event_limited_to_type',
								'value' => 'learndash_groups',
							],
							[
								'key'     => 'imm_calendar_event_limited_to_ids',
								'compare' => 'LIKE',
								'value'   => '"' . $group_id . '"',
							],
						],
					] ) )
				);
			}

			// Group leaders also see events created by them
			$post_ids_by_group_leader = get_posts( array_merge( $query_args, [
				'fields'      => 'ids',
				'post_status' => 'publish',
				'author'      => $user_id,
			] ) );

			if ( empty( $post_ids_with_group_leader_groups ) and empty( $post_ids_by_group_leader ) ) {
				return null;
			}

			$query_args['post__in'] = array_merge(
				$post_ids_with_group_leader_groups,
				$post_ids_by_group_leader
			);
		} elseif ( ! is_user_logged_in() ) {
			$post_ids_with_for_all = get_posts( array_merge( $query_args, [
				'fields'      => 'ids',
				'post_status' => 'publish',
				'meta_query'  => [
					[
						'key'   => 'imm_calendar_event_limited_to_type',
						'value' => 'all',
					],
				],
			] ) );

			if ( empty( $post_ids_with_for_all ) ) {
				return null;
			}

			$query_args['post__in'] = $post_ids_with_for_all;
		} else {
			// Students see events created by them

			$post_ids_created_by_student = get_posts( array_merge( $query_args, [
				'fields'      => 'ids',
				'post_status' => 'private',
				'author'      => $user_id,
			] ) );

			// And events for all

			$post_ids_with_for_all = get_posts( array_merge( $query_args, [
				'fields'      => 'ids',
				'post_status' => 'publish',
				'meta_query'  => [
					[
						'key'   => 'imm_calendar_event_limited_to_type',
						'value' => 'all',
					],
				],
			] ) );

			// And events assigned to them via user id

			$post_ids_with_student_id = get_posts(
				array_merge( $query_args, [
					'fields'      => 'ids',
					'post_status' => 'publish',
					'meta_query'  => [
						'relation' => 'AND',
						[
							'key'   => 'imm_calendar_event_limited_to_type',
							'value' => 'users',
						],
						[
							'key'     => 'imm_calendar_event_limited_to_ids',
							'compare' => 'LIKE',
							'value'   => '"' . $user_id . '"',
						],
					],
				] )
			);

			// And events assigned to them via their group

			$post_ids_with_student_groups = [];

			foreach ( learndash_get_users_group_ids( $user_id ) as $group_id ) {
				$post_ids_with_student_groups = array_merge(
					$post_ids_with_student_groups,
					get_posts( array_merge( $query_args, [
						'fields'      => 'ids',
						'post_status' => 'publish',
						'meta_query'  => [
							'relation' => 'AND',
							[
								'key'   => 'imm_calendar_event_limited_to_type',
								'value' => 'learndash_groups',
							],
							[
								'key'     => 'imm_calendar_event_limited_to_ids',
								'compare' => 'LIKE',
								'value'   => '"' . $group_id . '"',
							],
						],
					] ) )
				);
			}

			// And events assigned to them via their course

			$post_ids_with_student_courses = [];

			foreach ( learndash_get_user_course_access_list( $user_id ) as $course_id ) {
				$post_ids_with_student_courses = array_merge(
					$post_ids_with_student_courses,
					get_posts( array_merge( $query_args, [
						'fields'      => 'ids',
						'post_status' => 'publish',
						'meta_query'  => [
							'relation' => 'AND',
							[
								'key'   => 'imm_calendar_event_limited_to_type',
								'value' => 'learndash_courses',
							],
							[
								'key'     => 'imm_calendar_event_limited_to_ids',
								'compare' => 'LIKE',
								'value'   => '"' . $course_id . '"',
							],
						],
					] ) )
				);
			}

			// Merge all ids and find by ids (default meta search with joins is super slow)

			$post_ids = array_merge(
				$post_ids_created_by_student,
				$post_ids_with_for_all,
				$post_ids_with_student_id,
				$post_ids_with_student_groups,
				$post_ids_with_student_courses
			);

			if ( empty( $post_ids ) ) {
				return null;
			}

			$query_args['post__in'] = $post_ids;
		}

		return new WP_Query( $query_args );
	}

	/**
	 * @param int $user_id
	 *
	 * @return WP_Query|null
	 */
	protected function map_courses_and_quizzes_query( int $user_id ) {
		$course_ids = learndash_get_user_course_access_list( $user_id );

		if ( empty( $course_ids ) ) {
			return null;
		}

		// Prepare query

		$course_quiz_ids = [];

		foreach ( $course_ids as $course_id ) {
			$course_quiz_query = new WP_Query( [
				'post_type'      => 'sfwd-quiz',
				'posts_per_page' => -1,
				'meta_key'       => 'course_id',
				'meta_value'     => $course_id,
				'meta_compare'   => '=',
			] );

			if ( empty( $course_quiz_query->posts ) ) {
				continue;
			}

			$accessible_quizzes = array_filter( $course_quiz_query->posts, function ( $quiz ) use ( $user_id, $course_id ) {
				return is_quiz_accessable( $user_id, $quiz, false, $course_id );
			} );

			$course_quiz_ids = array_merge(
				$course_quiz_ids,
				wp_list_pluck( $accessible_quizzes, 'ID' )
			);
		}

		$post_ids = array_merge( $course_ids, $course_quiz_ids );

		if ( empty( $post_ids ) ) {
			return null;
		}

		return new WP_Query( [
			'post_type'      => [ 'sfwd-quiz', 'sfwd-courses' ],
			'post__in'       => $post_ids,
			'fields'         => 'ids',
			'post_status'    => 'publish',
			'posts_per_page' => -1,
			'meta_query'     => [
				[
					'key'     => 'imm_calendar_due_date',
					'compare' => 'EXISTS',
				],
			],
		] );
	}

	/**
	 * Returns lessons in the format [ 1 => 'Lesson 1 Name', 10 => 'Lesson 2 Name' ]
	 *
	 * @param int $user_id
	 *
	 * @return array
	 */
	protected function get_lessons_by_user( int $user_id ): array {
		if ( ! $this->user_is_student( $user_id ) ) {
			return [];
		}

		$lessons = [];

		foreach ( learndash_get_user_course_access_list( $user_id ) as $course_id ) {
			$lessons += wp_list_pluck( learndash_get_lesson_list( $course_id ), 'post_title', 'ID' );
		}

		return $lessons;
	}

	protected function user_is_student( int $user_id ): bool {
		return ! ( learndash_is_admin_user( $user_id ) or learndash_is_group_leader_user( $user_id ) );
	}
}
