<?php

/**
 * The file that defines the core plugin class
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://immerseus.com
 * @since      1.0.0
 * @package    Immerseus_Calendar
 * @subpackage Immerseus_Calendar/includes
 */

/**
 * The core plugin class.
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Immerseus_Calendar
 * @subpackage Immerseus_Calendar/includes
 * @author     Immerseus <info@immerseus.com>
 */
class Immerseus_Calendar {
	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Immerseus_Calendar_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'IMMERSEUS_CALENDAR_VERSION' ) ) {
			$this->version = IMMERSEUS_CALENDAR_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'immerseus-calendar';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 * Include the following files that make up the plugin:
	 * - Immerseus_Calendar_Loader. Orchestrates the hooks of the plugin.
	 * - Immerseus_Calendar_i18n. Defines internationalization functionality.
	 * - Immerseus_Calendar_Admin. Defines all hooks for the admin area.
	 * - Immerseus_Calendar_Public. Defines all hooks for the public side of the site.
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {
		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-i18n.php';

		/**
		 * The class responsible for mapping the events.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-events-mapper.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-upcoming-events-mapper.php';

		/**
		 * The class responsible for orchestrating the widget.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-widget.php';

		/**
		 * The class responsible for orchestrating the calendar event DTO.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-event-dto.php';

		/**
		 * The class responsible for orchestrating the calendar export.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-exporter.php';

		/**
		 * The class responsible for orchestrating the LearnDash achievements plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-immerseus-calendar-achievements-connector.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-immerseus-calendar-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-immerseus-calendar-public.php';

		$this->loader = new Immerseus_Calendar_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 * Uses the Immerseus_Calendar_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {
		$plugin_i18n = new Immerseus_Calendar_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new Immerseus_Calendar_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'init', $plugin_admin, 'schedule_event_reminders' );
		$this->loader->add_action( 'immerseus_calendar_remind_about_events', $plugin_admin, 'send_event_reminders' );

		$this->loader->add_action( 'init', $plugin_admin, 'setup_calendar_event_post_type' );
		$this->loader->add_action( 'pre_get_posts', $plugin_admin, 'limit_calendar_events_visibility' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'add_caps_to_group_leaders' );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'init_settings' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_menu' );

		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'add_calendar_event_meta_boxes' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_calendar_event_post_type_meta' );

		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'add_course_and_quiz_meta_boxes' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_course_and_quiz_meta' );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// LearnDash Achievements Plugin Connection

		$achievements_plugin_connector = new Immerseus_Calendar_Achievements_Connector();

		$this->loader->add_filter( 'learndash_achievements_triggers', $achievements_plugin_connector, 'add_trigger' );
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 * @since     1.0.0
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 * @since     1.0.0
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {
		$plugin_public = new Immerseus_Calendar_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'init', $plugin_public, 'add_shortcodes' );
		$this->loader->add_action( 'init', $plugin_public, 'add_blocks' );
		$this->loader->add_action( 'widgets_init', $plugin_public, 'add_widgets' );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		$this->loader->add_action( 'wp_ajax_imm_create_or_update_calendar_event', $plugin_public, 'create_or_update_event' );
		$this->loader->add_action( 'wp_ajax_imm_fetch_calendar_events', $plugin_public, 'fetch_events' );
		$this->loader->add_action( 'wp_ajax_nopriv_imm_fetch_calendar_events', $plugin_public, 'fetch_events' );
		$this->loader->add_action( 'wp_ajax_imm_delete_calendar_event', $plugin_public, 'delete_event' );

		// Events export

		$plugin_exporter = new Immerseus_Calendar_Exporter();

		$this->loader->add_filter( 'wp_headers', $plugin_exporter, 'set_headers' );
		$this->loader->add_action( 'template_redirect', $plugin_exporter, 'map_feed' );

		// LearnDash Achievements Plugin Connection

		$achievements_plugin_connector = new Immerseus_Calendar_Achievements_Connector();

		$this->loader->add_filter( 'learndash_lesson_completed', $achievements_plugin_connector, 'add_achievement' );
		$this->loader->add_filter( 'learndash_calendar_event_fields_on_frontend', $achievements_plugin_connector, 'add_field' );
		$this->loader->add_action( 'save_learndash_calendar_event_on_frontend', $achievements_plugin_connector, 'save_field', 10, 2 );
		$this->loader->add_action( 'wp_enqueue_scripts', $achievements_plugin_connector, 'enqueue_scripts' );
		$this->loader->add_action( 'create_immerseus_calendar_event', $achievements_plugin_connector, 'add_prop_to_calendar_object', 10, 2 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return    Immerseus_Calendar_Loader    Orchestrates the hooks of the plugin.
	 * @since     1.0.0
	 */
	public function get_loader() {
		return $this->loader;
	}
}
