<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://immerseus.com
 * @since      1.0.0
 * @package    Immerseus_Calendar
 * @subpackage Immerseus_Calendar/public
 */

/**
 * The public-facing functionality of the plugin.
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Immerseus_Calendar
 * @subpackage Immerseus_Calendar/public
 * @author     Immerseus <info@immerseus.com>
 */
class Immerseus_Calendar_Public {
	private $plugin_name;

	private $version;

	/**
	 * @var Immerseus_Calendar_Events_Mapper
	 */
	private $events_mapper;

	public function __construct( string $plugin_name, string $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		$this->events_mapper = new Immerseus_Calendar_Events_Mapper();
	}

	public function add_shortcodes() {
		add_shortcode( 'learndash_calendar', [ $this, 'calendar_shortcode_content' ] );
	}

	public function add_blocks() {
		// Skip block registration if Gutenberg is not enabled/merged.
		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}

		// Calendar

		wp_register_script(
			'immerseus-learndash-calendar-block-editor',
			IMMERSEUS_CALENDAR_URL . '/blocks/calendar/index.js',
			[ 'wp-blocks', 'wp-i18n', 'wp-element' ],
			$this->version
		);

		register_block_type( 'immerseus/learndash-calendar', [
			'editor_script'   => 'immerseus-learndash-calendar-block-editor',
			'render_callback' => [ $this, 'calendar_shortcode_content' ],
		] );
	}

	public function calendar_shortcode_content() {
		if ( is_admin() or ! shortcode_exists( 'learndash_calendar' ) ) {
			return '';
		}

		if ( ! get_option( 'imm_calendar_show_for_guests' ) and ! is_user_logged_in() ) {
			return '';
		}

		ob_start();

		$theme           = get_option( 'imm_calendar_theme', 'default' );
		$is_student      = $this->current_user_is_student();
		$export_feed_url = Immerseus_Calendar_Exporter::map_feed_url();

		include_once 'partials/immerseus-calendar-shortcode.php';

		return ob_get_clean();
	}

	protected function current_user_is_student(): bool {
		if ( ! is_user_logged_in() ) {
			return false;
		}

		return ! ( learndash_is_admin_user() or learndash_is_group_leader_user() );
	}

	public function add_widgets() {
		register_widget( 'Immerseus_Calendar_Widget' );
	}

	public function enqueue_styles() {
		if ( ! $this->include_widget_assets() and ! $this->include_shortcode_assets() ) {
			return;
		}

		if ( $this->include_widget_assets() ) {
			wp_enqueue_style(
				'simple-calendar',
				plugin_dir_url( __FILE__ ) . 'css/simple-calendar.css',
				[],
				$this->version
			);
		}

		if ( $this->include_shortcode_assets() ) {
			wp_enqueue_style( 'wp-color-picker', admin_url( 'css/color-picker.min.css' ) );

			wp_enqueue_style(
				'jquery-datetimepicker',
				plugin_dir_url( __FILE__ ) . 'css/jquery.datetimepicker.min.css',
				[],
				$this->version
			);

			wp_enqueue_style(
				'full-calendar',
				plugin_dir_url( __FILE__ ) . 'css/full-calendar.min.css',
				[],
				$this->version
			);
		}

		wp_enqueue_style(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'css/immerseus-calendar-public.css',
			[],
			$this->version
		);
	}

	protected function include_widget_assets(): bool {
		return is_active_widget( false, false, 'learndash_calendar' );
	}

	protected function include_shortcode_assets(): bool {
		return ( shortcode_exists( 'learndash_calendar' ) or has_block( 'immerseus/learndash-calendar' ) );
	}

	public function enqueue_scripts() {
		if ( ! $this->include_widget_assets() and ! $this->include_shortcode_assets() ) {
			return;
		}

		if ( $this->include_widget_assets() ) {
			wp_enqueue_script(
				'simple-calendar',
				plugin_dir_url( __FILE__ ) . 'js/jquery.simple-calendar.js',
				[ 'jquery' ],
				$this->version
			);

			wp_enqueue_script(
				$this->plugin_name . '-widget',
				plugin_dir_url( __FILE__ ) . 'js/immerseus-calendar-widget.js',
				[ 'jquery', 'simple-calendar' ],
				$this->version
			);
		}

		if ( $this->include_shortcode_assets() ) {
			wp_enqueue_script(
				'iris',
				admin_url( 'js/iris.min.js' ),
				[ 'jquery-ui-draggable', 'jquery-ui-slider', 'jquery-touch-punch' ]
			);

			wp_enqueue_script( 'wp-color-picker', admin_url( 'js/color-picker.min.js' ), [ 'iris', 'wp-i18n' ] );

			wp_enqueue_script(
				'jquery-datetimepicker',
				plugin_dir_url( __FILE__ ) . 'js/jquery.datetimepicker.full.min.js',
				[ 'jquery' ],
				$this->version
			);

			wp_enqueue_script(
				'moment',
				plugin_dir_url( __FILE__ ) . 'js/moment.js',
				[],
				$this->version
			);

			wp_enqueue_script(
				'parsley',
				plugin_dir_url( __FILE__ ) . 'js/parsley.min.js',
				[ 'jquery' ],
				$this->version
			);

			wp_enqueue_script( 'full-calendar', plugin_dir_url( __FILE__ ) . 'js/full-calendar.min.js', [], $this->version );

			wp_enqueue_script( 'full-calendar-locales', plugin_dir_url( __FILE__ ) . 'js/full-calendar-locales-all.min.js', [], $this->version );

			wp_enqueue_script( 'micromodal', plugin_dir_url( __FILE__ ) . 'js/micromodal.min.js', [], $this->version );
		}

		wp_enqueue_script( $this->plugin_name . '-shortcode', plugin_dir_url( __FILE__ ) . 'js/immerseus-calendar-shortcode.js', [
			'jquery',
			'jquery-datetimepicker',
		], $this->version );

		wp_localize_script(
			$this->plugin_name . '-shortcode',
			'imm_calendar',
			[
				'ajaxurl' => admin_url( 'admin-ajax.php' ),
				'nonce'   => wp_create_nonce( 'imm_calendar_event_action' ),
				'theme'   => get_option( 'imm_calendar_theme', 'default' ),
				'locale'   =>explode( '_',  get_locale() )[0],
			]
		);
	}

	/**
	 * Full calendar plugin passes "start" and "date" params, so sql query can be more picky.
	 * It may become useful if there will be too many events.
	 */
	public function fetch_events() {
		if ( ! isset( $_REQUEST['nonce'] ) or ! wp_verify_nonce( $_REQUEST['nonce'], 'imm_calendar_event_action' ) ) {
			wp_send_json( [] );
		}

		if ( ! get_option( 'imm_calendar_show_for_guests' ) and ! is_user_logged_in() ) {
			wp_send_json( [] );
		}

		wp_send_json(
			$this->events_mapper->map( get_current_user_id() )
		);
	}

	public function create_or_update_event() {
		if ( ! isset( $_REQUEST['nonce'] ) or ! wp_verify_nonce( $_REQUEST['nonce'], 'imm_calendar_event_action' ) ) {
			return;
		}

		$form_data = wp_list_pluck( $_POST['form_data'], 'value', 'name' );

		$title = sanitize_text_field( $form_data['title'] );

		if ( ! empty( $form_data['post_id'] ) ) {
			$post_id = intval( $form_data['post_id'] );
			$post    = get_post( $post_id );

			if ( is_null( $post ) or $post->post_author != get_current_user_id() ) {
				wp_send_json_error();
			}

			wp_update_post( [
				'ID'         => $post_id,
				'post_title' => $title,
			] );
		} else {
			$post_id = wp_insert_post( [
				'post_title'  => $title,
				'post_status' => 'private',
				'post_type'   => 'imm_calendar_event',
			] );
		}

		// Save meta

		$is_all_day = isset( $form_data['all_day'] );

		if ( ! empty( $form_data['link'] ) ) {
			update_post_meta(
				$post_id,
				'imm_calendar_event_link',
				wp_http_validate_url( $form_data['link'] )
			);
		}

		update_post_meta( $post_id, 'imm_calendar_event_all_day', $is_all_day );

		update_post_meta(
			$post_id,
			'imm_calendar_event_start_date',
			get_gmt_from_date( sanitize_text_field( $form_data['start_date'] ) )
		);

		if ( $is_all_day ) {
			delete_post_meta( $post_id, 'imm_calendar_event_end_date' );
		} else {
			update_post_meta(
				$post_id,
				'imm_calendar_event_end_date',
				get_gmt_from_date( sanitize_text_field( $form_data['end_date'] ) )
			);
		}

		update_post_meta(
			$post_id,
			'imm_calendar_event_background_color',
			sanitize_hex_color( $form_data['background_color'] ?: '#2270B1' )
		);

		update_post_meta(
			$post_id,
			'imm_calendar_event_text_color',
			sanitize_hex_color( $form_data['text_color'] ?: '#FFFFFF' )
		);

		do_action( 'save_learndash_calendar_event_on_frontend', $post_id, $form_data );

		// Send response

		wp_send_json_success( true );
	}

	public function delete_event() {
		if ( ! isset( $_REQUEST['nonce'] ) or ! wp_verify_nonce( $_REQUEST['nonce'], 'imm_calendar_event_action' ) ) {
			wp_send_json_error();
		}

		if ( ! $this->current_user_is_student() ) {
			wp_send_json_error();
		}

		$post_id = intval( $_REQUEST['post_id'] );
		$post    = get_post( $post_id );

		if ( is_null( $post ) or $post->post_author != get_current_user_id() ) {
			wp_send_json_error();
		}

		$result = wp_delete_post( $post_id );

		// Send response

		wp_send_json_success( $result );
	}
}
